#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from novaclient.tests.functional import base


class ResourceChecker(base.ClientTestBase):

    def runTest(self):
        pass

    def check(self):
        self.setUp()

        print("$ nova list --all-tenants")
        print(self.nova("list", params="--all-tenants"))
        print("\n")


if __name__ == "__main__":
    ResourceChecker().check()
