# Copyright 2011 OpenStack Foundation
# Copyright 2013 IBM Corp.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from novaclient.tests.unit.fixture_data import availability_zones as data
from novaclient.tests.unit.fixture_data import client
from novaclient.tests.unit import utils
from novaclient.tests.unit.v2 import fakes
from novaclient.v2 import availability_zones


class AvailabilityZoneTest(utils.FixturedTestCase):
    # NOTE(cyeoh): import shell here so the V3 version of
    # this class can inherit off the v3 version of shell
    from novaclient.v2 import shell  # noqa

    data_fixture_class = data.V1

    scenarios = [('original', {'client_fixture_class': client.V1}),
                 ('session', {'client_fixture_class': client.SessionV1})]

    def setUp(self):
        super(AvailabilityZoneTest, self).setUp()
        self.availability_zone_type = self._get_availability_zone_type()

    def _get_availability_zone_type(self):
        return availability_zones.AvailabilityZone

    def _assertZone(self, zone, name, status):
        self.assertEqual(zone.zoneName, name)
        self.assertEqual(zone.zoneState, status)

    def test_list_availability_zone(self):
        zones = self.cs.availability_zones.list(detailed=False)
        self.assert_request_id(zones, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('GET', '/os-availability-zone')

        for zone in zones:
            self.assertIsInstance(zone, self.availability_zone_type)

        self.assertEqual(2, len(zones))

        l0 = ['zone-1', 'available']
        l1 = ['zone-2', 'not available']

        z0 = self.shell._treeizeAvailabilityZone(zones[0])
        z1 = self.shell._treeizeAvailabilityZone(zones[1])

        self.assertEqual((1, 1), (len(z0), len(z1)))

        self._assertZone(z0[0], l0[0], l0[1])
        self._assertZone(z1[0], l1[0], l1[1])

    def test_detail_availability_zone(self):
        zones = self.cs.availability_zones.list(detailed=True)
        self.assert_request_id(zones, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('GET', '/os-availability-zone/detail')

        for zone in zones:
            self.assertIsInstance(zone, self.availability_zone_type)

        self.assertEqual(3, len(zones))

        l0 = ['zone-1', 'available']
        l1 = ['|- fake_host-1', '']
        l2 = ['| |- nova-compute', 'enabled :-) 2012-12-26 14:45:25']
        l3 = ['internal', 'available']
        l4 = ['|- fake_host-1', '']
        l5 = ['| |- nova-sched', 'enabled :-) 2012-12-26 14:45:25']
        l6 = ['|- fake_host-2', '']
        l7 = ['| |- nova-network', 'enabled XXX 2012-12-26 14:45:24']
        l8 = ['zone-2', 'not available']

        z0 = self.shell._treeizeAvailabilityZone(zones[0])
        z1 = self.shell._treeizeAvailabilityZone(zones[1])
        z2 = self.shell._treeizeAvailabilityZone(zones[2])

        self.assertEqual((3, 5, 1), (len(z0), len(z1), len(z2)))

        self._assertZone(z0[0], l0[0], l0[1])
        self._assertZone(z0[1], l1[0], l1[1])
        self._assertZone(z0[2], l2[0], l2[1])
        self._assertZone(z1[0], l3[0], l3[1])
        self._assertZone(z1[1], l4[0], l4[1])
        self._assertZone(z1[2], l5[0], l5[1])
        self._assertZone(z1[3], l6[0], l6[1])
        self._assertZone(z1[4], l7[0], l7[1])
        self._assertZone(z2[0], l8[0], l8[1])
